#
# Copyright (c) 1997 Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#  This product includes software developed by the Computer Systems
#  Engineering Group at Lawrence Berkeley Laboratory.
# 4. Neither the name of the University nor of the Laboratory may be used
#    to endorse or promote products derived from this software without
#    specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.

# Testing the http objects.

# Setup the class definitions and random objects.
source httpModel.tcl



proc finish file {
        global env
	set f [open temp.rands w]
	puts $f "TitleText: $file"
	puts $f "Device: Postscript"
	
	exec rm -f temp.p temp.d 
	exec touch temp.d temp.p
	#
	# split queue/drop events into two separate files.
	# we don't bother checking for the link we're interested in
	# since we know only such events are in our trace file
	#
	exec awk {
		{
			if (($1 == "+" || $1 == "-" ) && \
			    ($5 == "tcp" || $5 == "ack"))\
					print $2, $8 + ($11 % 90) * 0.01
		}
	} out.tr > temp.p
	exec awk {
		{
			if ($1 == "d")
				print $2, $8 + ($11 % 90) * 0.01
		}
	} out.tr > temp.d

	puts $f \"packets
	flush $f
	exec cat temp.p >@ $f
	flush $f
	# insert dummy data sets so we get X's for marks in data-set 4
	puts $f [format "\n\"skip-1\n0 1\n\n\"skip-2\n0 1\n\n"]

	puts $f \"drops
	flush $f
	#
	# Repeat the first line twice in the drops file because
	# often we have only one drop and xgraph won't print marks
	# for data sets with only one point.
	#
	exec head -1 temp.d >@ $f
	exec cat temp.d >@ $f
	close $f
	exec xgraph -display $env(DISPLAY) -bb -tk -nl -m -x time -y packet temp.rands &
	
	exit 0
}



Simulator instproc openTrace { stopTime testName } {
    exec rm -f out.tr temp.rands
    set traceFile [open out.tr w]
    $self at $stopTime \
	    "close $traceFile ; finish $testName"
    return $traceFile
}



# Create the simulator.
set ns [new Simulator]

# Create the network topology.
set node1 [$ns node]
set node2 [$ns node]
$ns duplex-link $node1 $node2 10Mb 10ms DropTail

# Create the www objects and cdf's.
initCdfs $ns
set wwwServer [new WWWServer $node1 $ns]
# Start 5 clients.
for {set i 0} {$i < 5} {incr i} {
    set wwwClient($i) [new WWWClient $node2 $ns]
    $ns at 0.0 "$wwwClient($i) start $wwwServer"
}

$wwwClient(0) statDump 20

# The background FTP source.
set ftp(1) [new Source/FTP]
set tcp(1) [new Agent/TCP]
set tcpSink(1) [new Agent/TCPSink]
$tcp(1) set fid_ 0 
$tcpSink(1) set fid_ 0
$tcp(1) set type FTP
$tcp(1) set overhead_ 0.005
$ftp(1) set agent_ $tcp(1)
# $ns at 0 "$ftp(1) start"

$ns attach-agent $node1 $tcp(1)
$ns attach-agent $node2 $tcpSink(1)
$ns connect $tcp(1) $tcpSink(1)
$tcp(1) set window_ 57


# Test the objects.
$ns trace-queue $node1 $node2 [$ns openTrace 60.1 http]
$ns run
